/****************************************************************************************************

RepRapFirmware - Platform

Platform contains all the code and definitions to deal with machine-dependent things such as control
pins, bed area, number of extruders, tolerable accelerations and speeds and so on.

No definitions that are system-independent should go in here.  Put them in Configuration.h.

-----------------------------------------------------------------------------------------------------

Version 0.3

28 August 2013

Adrian Bowyer
RepRap Professional Ltd
http://reprappro.com

Licence: GPL

 ****************************************************************************************************/


#ifndef LYNXMOD_H
#define LYNXMOD_H

// Language-specific includes

#include <RepRapFirmware.h>
#if SUPPORT_LYNXMOD
// Platform-specific includes
#include <Platform/RepRap.h>
#include <Platform/Platform.h>
#include <GCodes/GCodes.h>

#include "Hardware/IoPorts.h"

//#include "Heating/Heat.h"
//#include "Platform/Logger.h"

#if SUPPORT_SPI_SENSORS

#include <Hardware/SharedSpi/SharedSpiClient.h>
#include <Heating/TemperatureError.h>		// for result codes

class LynxMod INHERIT_OBJECT_MODEL
{

private :
	int16_t toolID = -1, rackID = -1, optionsID[10] = {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1};

	const SpiMode Nano_SpiMode = SPI_MODE_1;
	const uint32_t Nano_Frequency = 1000000;
	// Define the minimum interval between readings. The MAX31865 needs 62.5ms in 50Hz filter mode.
	const uint32_t MinimumReadInterval = 250;		// minimum interval between reads, in milliseconds

	IoPort port;
	SharedSpiClient device;
	uint32_t lastReadingTime;
	TemperatureError lastResult;
	uint8_t cr0;


	unsigned long flash_cpt=0, test_cpt=0, new_cpt=0, trame_col=0;
	char trame_ver=0;
	bool flash = false, test_leds = false, new_color = false;

	struct LightType {
		char name[9];
		unsigned long color;
	} listAmbiances[24];

	TemperatureError DoSpiTransaction(const uint8_t dataOut[], size_t nbytes, uint32_t& rslt) const noexcept
		pre(nbytes <= 8);
	GCodeResult FinishConfiguring(bool changed, const StringRef& reply) noexcept;
	TemperatureError TryInitLED() const noexcept;

protected:
	DECLARE_OBJECT_MODEL

public :

	LynxMod() noexcept;
	void Init() noexcept;
	void Spin() noexcept {}; 															// Routine générale
	bool IsPorteOuverte() noexcept { return false; };
	int32_t GetRackID() { return rackID; }
	int32_t GetToolID() { return toolID; }
	int32_t GetOptionID(unsigned int optionIndex) { return optionsID[optionIndex]; }

	// Routine générale
	GCodeResult Configure(GCodeBuffer& gb, const StringRef& reply, bool& changed) THROWS(GCodeException);
	bool ConfigurePort(GCodeBuffer& gb, const StringRef& reply, PinAccess access, bool& seen) THROWS(GCodeException);

	GCodeResult LynxM968(GCodeBuffer& gb, const StringRef& reply) THROWS(GCodeException) {return GCodeResult::warningNotSupported; };

	GCodeResult LynxM969(GCodeBuffer& gb, const StringRef& reply) THROWS(GCodeException);
	GCodeResult LynxM969(int val, int pval, float* cval, const StringRef& reply);

	void Diagnostics(MessageType mtype) noexcept {};
};

#endif //SUPPORT_SPI_SENSORS
#endif //SUPPORT_LYNXMOD
#endif //LYNXMOD_H
